#include <stdlib.h>
#include "mex.h"
#include "svm.h"

#define NUM_OF_RETURN_FIELD 8

void model_to_matlab_structure(mxArray *plhs[], int num_of_feature, struct svm_model *model){

	int i, j, n;
	double *ptr;
	mxArray *cell_array_ptr, **rhs;

	rhs = (mxArray **)mxMalloc(sizeof(mxArray *)*NUM_OF_RETURN_FIELD);	

	// sv_coef
	rhs[0] = mxCreateDoubleMatrix(model->l, model->nr_class-1, mxREAL);
	ptr = mxGetPr(rhs[0]);
	for(i = 0; i < model->nr_class-1; i++)
		for(j = 0; j < model->l; j++)
			ptr[(i*(model->l))+j] = model->sv_coef[i][j];
	// SVs
	rhs[1] = mxCreateDoubleMatrix(model->l, num_of_feature, mxREAL);
	ptr = mxGetPr(rhs[1]);

	for(i = 0; i < model->l; i++)
	{
		j = 0;
		while(model->SV[i][j].index != -1)
		{
			ptr[(model->SV[i][j].index-1)*model->l+i] = model->SV[i][j].value;
			j++;	
		}
	}

	n = model->nr_class*(model->nr_class-1)/2;
	// rho
	rhs[2] = mxCreateDoubleMatrix(n, 1, mxREAL);
	ptr = mxGetPr(rhs[2]);
	for(i = 0; i < n; i++)
		ptr[i] = model->rho[i];

	// Parameters
	rhs[3] = mxCreateDoubleMatrix(5, 1, mxREAL); 
	ptr = mxGetPr(rhs[3]);
	ptr[0] = model->param.svm_type;
	ptr[1] = model->param.kernel_type;
	ptr[2] = model->param.degree;
	ptr[3] = model->param.gamma;
	ptr[4] = model->param.coef0;

	// nSV
	rhs[4] = mxCreateDoubleMatrix(model->nr_class, 1, mxREAL);
	ptr = mxGetPr(rhs[4]);
	for(i = 0; i < model->nr_class; i++)
		ptr[i] = model->nSV[i];

	// Label
	rhs[5] = mxCreateDoubleMatrix(model->nr_class, 1, mxREAL);
	ptr = mxGetPr(rhs[5]);
	for(i = 0; i < model->nr_class; i++)
		ptr[i] = model->label[i];
	
	// probA, probB
	if(model->probA != NULL && model->probB != NULL)
	{
		rhs[6] = mxCreateDoubleMatrix(n, 1, mxREAL);
		ptr = mxGetPr(rhs[6]);
		for(i = 0; i < n; i++)
			ptr[i] = model->probA[i];

		rhs[7] = mxCreateDoubleMatrix(n, 1, mxREAL);
		ptr = mxGetPr(rhs[7]);
		for(i = 0; i < n; i++)
			ptr[i] = model->probB[i];
	}
	else 
	{
		rhs[6] = mxCreateDoubleMatrix(0, 0, mxREAL);
		rhs[7] = mxCreateDoubleMatrix(0, 0, mxREAL);
	}
	
	/* Create a nrhs x 1 cell mxArray. */ 
	cell_array_ptr = mxCreateCellMatrix(NUM_OF_RETURN_FIELD, 1);

	/* Fill cell matrix with input arguments */
	for(i = 0; i < NUM_OF_RETURN_FIELD; i++)
	{
		mxSetCell(cell_array_ptr,i,mxDuplicateArray(rhs[i]));
	}
	/* return */
	plhs[0] = cell_array_ptr;

	return ;
}
